import { promiseWithResolvers } from '../common/promiseWithResolvers.js';
export { IterableChannel };
class IterableChannel {
    #isClosed = false;
    #nextIteration = promiseWithResolvers();
    #currentValue;
    put(value) {
        if (!this.#isClosed) {
            (async () => {
                this.#currentValue = value;
                await undefined; // Deferring to the next microtick so that an attempt to pull the a value before making multiple rapid synchronous calls to `put()` will make that pull ultimately yield only the last value that was put - instead of the first one as were if this otherwise wasn't deferred.
                this.#nextIteration.resolve({ done: false, value: this.#currentValue });
                this.#nextIteration = promiseWithResolvers();
            })();
        }
    }
    close() {
        this.#isClosed = true;
        this.#nextIteration.resolve({ done: true, value: undefined });
    }
    values = {
        value: (() => {
            const self = this;
            return {
                get current() {
                    return self.#currentValue;
                },
            };
        })(),
        [Symbol.asyncIterator]: () => {
            const whenIteratorClosed = promiseWithResolvers();
            return {
                next: () => {
                    return Promise.race([this.#nextIteration.promise, whenIteratorClosed.promise]);
                },
                return: async () => {
                    whenIteratorClosed.resolve({ done: true, value: undefined });
                    return { done: true, value: undefined };
                },
            };
        },
    };
}
//# sourceMappingURL=IterableChannel.js.map